/*  Routines for manipulating B-splines.  These are intended for use with
 *  S or S-PLUS or R.
 *
 *     Copyright (C) 2015 Michel Grzebyk.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * These functions are distributed in the hope that they will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, a copy is available at
 *  http://www.r-project.org/Licenses/
 *
 * The routines are loosely based on the pseudo-code in Schumacher (Wiley,
 * 1981) and the CMLIB library DBSPLINES.
 */

#include <R.h>
#include <Rinternals.h>

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext ("splines", String)
#else
#define _(String) (String)
#endif

#ifndef EVAL_SPLINEPARAM
#include "SplineParam.h"
#endif

SEXP eval_espline_basis(SEXP knots, SEXP order, SEXP Matrices, SEXP intercept, SEXP xvals)
{
	/* evaluate the non-zero B-spline basis functions at xvals, using matrices generated by packag::orthogonalsplinbasis */
	/* Matrices is a vectorized array of dim order X nbases X number_of_intervales(knots)
  where nbases is the number of bases of the non integrated, non derived splines
	 */
	R_len_t i, j, k, nknots, theorder, nbases, nx;
	R_len_t theinterval, firstbasis, mfl;
	double *rknots, *rMatrices, *rxvals, *rbases;
	SEXP bases;
	SEXP dims;
	double temp, *U, u, outer_val;

	outer_val = 0.0;

	PROTECT(knots = coerceVector(knots, REALSXP));
	PROTECT(order = coerceVector(order, INTSXP));
	PROTECT(intercept = coerceVector(intercept, INTSXP));
	PROTECT(Matrices = coerceVector(Matrices, REALSXP));
	PROTECT(xvals = coerceVector(xvals, REALSXP));


	rknots = REAL(knots);
	nknots = length(knots);
	theorder = INTEGER(order)[0];

	dims = getAttrib(Matrices, R_DimSymbol);
	if( LENGTH(dims) < 3 ){
		error("'Matrices' must be an array with 3 dim");
	}
	nbases = INTEGER(dims)[1];

	rxvals = REAL(xvals);
	nx = length(xvals);

	firstbasis = (INTEGER(intercept)[0]==0);
	rMatrices = REAL(Matrices);

	PROTECT(bases = allocMatrix(REALSXP, nx, nbases-firstbasis));
	rbases = REAL(bases);


	U = (double *) R_alloc( theorder, sizeof(double));


	U[0]=1.0;
	for(i = 0; i < nx; i++) {
		EVALUATE_one_espline_basis (rxvals[i], rbases, i + nx *) ;
	}

	UNPROTECT(6);
	return(bases);
}



